/*********************************************************************
 * NAN - Native Abstractions for Node.js
 *
 * Copyright (c) 2018 NAN contributors
 *
 * MIT License <https://github.com/nodejs/nan/blob/master/LICENSE.md>
 ********************************************************************/

#ifndef TEST_CPP_MULTIFILE2_H_
#define TEST_CPP_MULTIFILE2_H_

#include <nan.h>

using namespace Nan;  // NOLINT(build/namespaces)

NAN_METHOD(ReturnString);

#endif  // TEST_CPP_MULTIFILE2_H_
